@@ -1,10 +1,12 @@
 # Matplotlib documentation build configuration file, created by
 # sphinx-quickstart on Fri May  2 12:33:25 2008.
 #
-# This file is execfile()d with the current directory set to its containing dir.
+# This file is execfile()d with the current directory set to its containing
+# dir.
 #
 # The contents of this file are pickled, so don't put values in the namespace
-# that aren't pickleable (module imports are okay, they're removed automatically).
+# that aren't pickleable (module imports are okay, they're removed
+# automatically).
 #
 # All configuration values have a default value; values that are commented out
 # serve to show the default value.
@@ -17,10 +19,21 @@ import sys
 import warnings
 
 import matplotlib
-from matplotlib._api import MatplotlibDeprecationWarning
 import sphinx
 
 from datetime import datetime
+import time
+
+# Release mode enables optimizations and other related options.
+is_release_build = tags.has('release')  # noqa
+
+# are we running circle CI?
+CIRCLECI = 'CIRCLECI' in os.environ
+
+# Parse year using SOURCE_DATE_EPOCH, falling back to current time.
+# https://reproducible-builds.org/specs/source-date-epoch/
+sourceyear = datetime.utcfromtimestamp(
+    int(os.environ.get('SOURCE_DATE_EPOCH', time.time()))).year
 
 # If your extensions are in another directory, add it here. If the directory
 # is relative to the documentation root, use os.path.abspath to make it
@@ -49,7 +62,6 @@ extensions = [
     'sphinx.ext.inheritance_diagram',
     'sphinx.ext.intersphinx',
     'sphinx.ext.ifconfig',
-    'sphinx.ext.viewcode',
     'IPython.sphinxext.ipython_console_highlighting',
     'IPython.sphinxext.ipython_directive',
     'numpydoc',  # Needs to be loaded *after* autodoc.
@@ -65,14 +77,15 @@ extensions = [
     'sphinxext.skip_deprecated',
     'sphinxext.redirect_from',
     'sphinx_copybutton',
+    'sphinx_panels',
 ]
 
 exclude_patterns = [
     'api/prev_api_changes/api_changes_*/*',
-    # Be sure to update users/whats_new.rst:
-    'users/prev_whats_new/whats_new_3.4.0.rst',
 ]
 
+panels_add_bootstrap_css = False
+
 
 def _check_dependencies():
     names = {
@@ -81,6 +94,7 @@ def _check_dependencies():
         "matplotlib": 'matplotlib',
         "numpydoc": 'numpydoc',
         "PIL.Image": 'pillow',
+        "pydata_sphinx_theme": 'pydata_sphinx_theme',
         "sphinx_copybutton": 'sphinx_copybutton',
         "sphinx_gallery": 'sphinx_gallery',
         "sphinxcontrib.inkscapeconverter": 'sphinxcontrib-svg2pdfconverter',
@@ -107,6 +121,7 @@ _check_dependencies()
 # gallery_order.py from the sphinxext folder provides the classes that
 # allow custom ordering of sections and subsections of the gallery
 import sphinxext.gallery_order as gallery_order
+
 # The following import is only necessary to monkey patch the signature later on
 from sphinx_gallery import gen_rst
 
@@ -117,7 +132,7 @@ autosummary_generate = True
 
 # we should ignore warnings coming from importing deprecated modules for
 # autodoc purposes, as this will disappear automatically when they are removed
-warnings.filterwarnings('ignore', category=MatplotlibDeprecationWarning,
+warnings.filterwarnings('ignore', category=DeprecationWarning,
                         module='importlib',  # used by sphinx.autodoc.importer
                         message=r'(\n|.)*module was deprecated.*')
 
@@ -126,7 +141,7 @@ autodoc_default_options = {'members': None, 'undoc-members': None}
 
 # make sure to ignore warnings that stem from simply inspecting deprecated
 # class-level attributes
-warnings.filterwarnings('ignore', category=MatplotlibDeprecationWarning,
+warnings.filterwarnings('ignore', category=DeprecationWarning,
                         module='sphinx.util.inspect')
 
 # missing-references names matches sphinx>=3 behavior, so we can't be nitpicky
@@ -146,18 +161,20 @@ intersphinx_mapping = {
     'pytest': ('https://pytest.org/en/stable/', None),
     'python': ('https://docs.python.org/3/', None),
     'scipy': ('https://docs.scipy.org/doc/scipy/reference/', None),
+    'tornado': ('https://www.tornadoweb.org/en/stable/', None),
 }
 
 
 # Sphinx gallery configuration
+
 sphinx_gallery_conf = {
-    'examples_dirs': ['../examples', '../tutorials'],
+    'examples_dirs': ['../examples', '../tutorials', '../plot_types'],
     'filename_pattern': '^((?!sgskip).)*$',
-    'gallery_dirs': ['gallery', 'tutorials'],
+    'gallery_dirs': ['gallery', 'tutorials', 'plot_types'],
     'doc_module': ('matplotlib', 'mpl_toolkits'),
     'reference_url': {
         'matplotlib': None,
-        'numpy': 'https://docs.scipy.org/doc/numpy/',
+        'numpy': 'https://numpy.org/doc/stable/',
         'scipy': 'https://docs.scipy.org/doc/scipy/reference/',
     },
     'backreferences_dir': Path('api') / Path('_as_gen'),
@@ -166,11 +183,15 @@ sphinx_gallery_conf = {
     'remove_config_comments': True,
     'min_reported_time': 1,
     'thumbnail_size': (320, 224),
-    'compress_images': ('thumbnails', 'images'),
+    # Compression is a significant effort that we skip for local and CI builds.
+    'compress_images': ('thumbnails', 'images') if is_release_build else (),
     'matplotlib_animations': True,
+    'image_srcset': ["2x"],
+    'junit': '../test-results/sphinx-gallery/junit.xml' if CIRCLECI else '',
 }
 
-plot_gallery = 'True'
+mathmpl_fontsize = 11.0
+mathmpl_srcset = ['2x']
 
 # Monkey-patching gallery signature to include search keywords
 gen_rst.SPHX_GLR_SIG = """\n
@@ -192,28 +213,27 @@ source_suffix = '.rst'
 source_encoding = "utf-8"
 
 # The master toctree document.
-master_doc = 'contents'
+master_doc = 'users/index'
 
 # General substitutions.
 try:
     SHA = subprocess.check_output(
         ['git', 'describe', '--dirty']).decode('utf-8').strip()
-# Catch the case where git is not installed locally, and use the versioneer
+# Catch the case where git is not installed locally, and use the setuptools_scm
 # version number instead
 except (subprocess.CalledProcessError, FileNotFoundError):
     SHA = matplotlib.__version__
 
 html_context = {
-    'sha': SHA,
-    # This will disable any analytics in the HTML templates (currently Google
-    # Analytics.)
-    'include_analytics': False,
+    "sha": SHA,
 }
 
 project = 'Matplotlib'
-copyright = ('2002 - 2012 John Hunter, Darren Dale, Eric Firing, '
-             'Michael Droettboom and the Matplotlib development '
-             f'team; 2012 - {datetime.now().year} The Matplotlib development team')
+copyright = (
+    '2002 - 2012 John Hunter, Darren Dale, Eric Firing, Michael Droettboom '
+    'and the Matplotlib development team; '
+    f'2012 - {sourceyear} The Matplotlib development team'
+)
 
 
 # The default replacements for |version| and |release|, also used in various
@@ -227,7 +247,7 @@ release = version
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
-#today = ''
+# today = ''
 # Else, today_fmt is used as the format for a strftime call.
 today_fmt = '%B %d, %Y'
 
@@ -235,15 +255,15 @@ today_fmt = '%B %d, %Y'
 unused_docs = []
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
-#add_function_parentheses = True
+# add_function_parentheses = True
 
 # If true, the current module name will be prepended to all description
 # unit titles (such as .. function::).
-#add_module_names = True
+# add_module_names = True
 
 # If true, sectionauthor and moduleauthor directives will be shown in the
 # output. They are ignored by default.
-#show_authors = False
+# show_authors = False
 
 # The name of the Pygments (syntax highlighting) style to use.
 pygments_style = 'sphinx'
@@ -265,16 +285,32 @@ github_project_url = "https://github.com/matplotlib/matplotlib/"
 # The style sheet to use for HTML and HTML Help pages. A file of that name
 # must exist either in Sphinx' static/ path, or in one of the custom paths
 # given in html_static_path.
-#html_style = 'matplotlib.css'
-html_style = f'mpl.css?{SHA}'
+# html_style = 'matplotlib.css'
+# html_style = f"mpl.css?{SHA}"
+html_css_files = [
+    f"mpl.css?{SHA}",
+]
+
+html_theme = "mpl_sphinx_theme"
 
 # The name for this set of Sphinx documents.  If None, it defaults to
 # "<project> v<release> documentation".
-#html_title = None
+# html_title = None
 
 # The name of an image file (within the static path) to place at the top of
 # the sidebar.
-#html_logo = 'logo.png'
+html_logo = "_static/logo2.svg"
+html_theme_options = {
+    "native_site": True,
+    "logo_link": "index",
+    # collapse_navigation in pydata-sphinx-theme is slow, so skipped for local
+    # and CI builds https://github.com/pydata/pydata-sphinx-theme/pull/386
+    "collapse_navigation": not is_release_build,
+    "show_prev_next": False,
+}
+include_analytics = is_release_build
+if include_analytics:
+    html_theme_options["google_analytics_id"] = "UA-55954603-1"
 
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
@@ -293,23 +329,29 @@ html_last_updated_fmt = '%b %d, %Y'
 html_index = 'index.html'
 
 # Custom sidebar templates, maps document names to template names.
-#html_sidebars = {}
+# html_sidebars = {}
 
 # Custom sidebar templates, maps page names to templates.
 html_sidebars = {
-    'index': [
+    "index": [
+        'search-field.html',
         # 'sidebar_announcement.html',
-        'sidebar_versions.html',
-        'donate_sidebar.html'],
-    '**': ['localtoc.html', 'pagesource.html']
+        "sidebar_versions.html",
+        "cheatsheet_sidebar.html",
+        "donate_sidebar.html",
+    ],
+    # '**': ['localtoc.html', 'pagesource.html']
 }
 
-# If false, no module index is generated.
-#html_use_modindex = True
-html_domain_indices = ["py-modindex"]
+# If true, add an index to the HTML documents.
+html_use_index = False
+
+# If true, generate domain-specific indices in addition to the general index.
+# For e.g. the Python domain, this is the global module index.
+html_domain_index = False
 
 # If true, the reST sources are included in the HTML build as _sources/<name>.
-#html_copy_source = True
+# html_copy_source = True
 
 # If true, an OpenSearch description file will be output, and all pages will
 # contain a <link> tag referring to it.
@@ -330,8 +372,10 @@ html_favicon = '_static/favicon.ico'
 # The paper size ('letter' or 'a4').
 latex_paper_size = 'letter'
 
-# Grouping the document tree into LaTeX files. List of tuples
-# (source start file, target name, title, author, document class [howto/manual]).
+# Grouping the document tree into LaTeX files.
+# List of tuples:
+#   (source start file, target name, title, author,
+#    document class [howto/manual])
 
 latex_documents = [
     ('contents', 'Matplotlib.tex', 'Matplotlib',
@@ -485,9 +529,92 @@ graphviz_dot = shutil.which('dot')
 # graphviz_output_format = 'svg'
 
 
+def reduce_plot_formats(app):
+    # Fox CI and local builds, we don't need all the default plot formats, so
+    # only generate the directly useful one for the current builder.
+    if app.builder.name == 'html':
+        keep = 'png'
+    elif app.builder.name == 'latex':
+        keep = 'pdf'
+    else:
+        return
+    app.config.plot_formats = [entry
+                               for entry in app.config.plot_formats
+                               if entry[0] == keep]
+
+
 def setup(app):
     if any(st in version for st in ('post', 'alpha', 'beta')):
         bld_type = 'dev'
     else:
         bld_type = 'rel'
     app.add_config_value('releaselevel', bld_type, 'env')
+
+    if not is_release_build:
+        app.connect('builder-inited', reduce_plot_formats)
+
+# -----------------------------------------------------------------------------
+# Source code links
+# -----------------------------------------------------------------------------
+link_github = True
+# You can add build old with link_github = False
+
+if link_github:
+    import inspect
+    from packaging.version import parse
+
+    extensions.append('sphinx.ext.linkcode')
+
+    def linkcode_resolve(domain, info):
+        """
+        Determine the URL corresponding to Python object
+        """
+        if domain != 'py':
+            return None
+
+        modname = info['module']
+        fullname = info['fullname']
+
+        submod = sys.modules.get(modname)
+        if submod is None:
+            return None
+
+        obj = submod
+        for part in fullname.split('.'):
+            try:
+                obj = getattr(obj, part)
+            except AttributeError:
+                return None
+
+        try:
+            fn = inspect.getsourcefile(obj)
+        except TypeError:
+            fn = None
+        if not fn or fn.endswith('__init__.py'):
+            try:
+                fn = inspect.getsourcefile(sys.modules[obj.__module__])
+            except (TypeError, AttributeError, KeyError):
+                fn = None
+        if not fn:
+            return None
+
+        try:
+            source, lineno = inspect.getsourcelines(obj)
+        except (OSError, TypeError):
+            lineno = None
+
+        linespec = (f"#L{lineno:d}-L{lineno + len(source) - 1:d}"
+                    if lineno else "")
+
+        startdir = Path(matplotlib.__file__).parent.parent
+        fn = os.path.relpath(fn, start=startdir).replace(os.path.sep, '/')
+
+        if not fn.startswith(('matplotlib/', 'mpl_toolkits/')):
+            return None
+
+        version = parse(matplotlib.__version__)
+        tag = 'master' if version.is_devrelease else f'v{version.public}'
+        return ("https://github.com/matplotlib/matplotlib/blob"
+                f"/{tag}/lib/{fn}{linespec}")
+else:
+    extensions.append('sphinx.ext.viewcode')
