@@ -1,20 +1,17 @@
-'''
+"""
 ========================
 3D surface (solid color)
 ========================
 
 Demonstrates a very basic plot of a 3D surface using a solid color.
-'''
-
-# This import registers the 3D projection, but is otherwise unused.
-from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
+"""
 
 import matplotlib.pyplot as plt
 import numpy as np
 
 
 fig = plt.figure()
-ax = fig.add_subplot(111, projection='3d')
+ax = fig.add_subplot(projection='3d')
 
 # Make data
 u = np.linspace(0, 2 * np.pi, 100)
@@ -24,6 +21,6 @@ y = 10 * np.outer(np.sin(u), np.sin(v))
 z = 10 * np.outer(np.ones(np.size(u)), np.cos(v))
 
 # Plot the surface
-ax.plot_surface(x, y, z, color='b')
+ax.plot_surface(x, y, z)
 
 plt.show()
