@@ -7,6 +7,7 @@ import pytest
 
 import matplotlib as mpl
 import matplotlib.pyplot as plt
+from matplotlib.backend_bases import MouseEvent
 import matplotlib.collections as mcollections
 import matplotlib.colors as mcolors
 import matplotlib.transforms as mtransforms
@@ -303,7 +304,7 @@ def test_add_collection():
     assert ax.dataLim.bounds == bounds
 
 
-@pytest.mark.style('mpl20')
+@mpl.style.context('mpl20')
 @check_figures_equal(extensions=['png'])
 def test_collection_log_datalim(fig_test, fig_ref):
     # Data limits should respect the minimum x/y when using log scale.
@@ -514,7 +515,7 @@ def test_pandas_indexing(pd):
     Collection(antialiaseds=aa)
 
 
-@pytest.mark.style('default')
+@mpl.style.context('default')
 def test_lslw_bcast():
     col = mcollections.PathCollection([])
     col.set_linestyles(['-', '-'])
@@ -528,7 +529,7 @@ def test_lslw_bcast():
     assert (col.get_linewidths() == [1, 2, 3]).all()
 
 
-@pytest.mark.style('default')
+@mpl.style.context('default')
 def test_capstyle():
     col = mcollections.PathCollection([], capstyle='round')
     assert col.get_capstyle() == 'round'
@@ -536,7 +537,7 @@ def test_capstyle():
     assert col.get_capstyle() == 'butt'
 
 
-@pytest.mark.style('default')
+@mpl.style.context('default')
 def test_joinstyle():
     col = mcollections.PathCollection([], joinstyle='round')
     assert col.get_joinstyle() == 'round'
@@ -677,6 +678,22 @@ def test_collection_set_verts_array():
         assert np.array_equal(ap._codes, atp._codes)
 
 
+def test_collection_set_array():
+    vals = [*range(10)]
+
+    # Test set_array with list
+    c = Collection()
+    c.set_array(vals)
+
+    # Test set_array with wrong dtype
+    with pytest.raises(TypeError, match="^Image data of dtype"):
+        c.set_array("wrong_input")
+
+    # Test if array kwarg is copied
+    vals[5] = 45
+    assert np.not_equal(vals, c.get_array()).any()
+
+
 def test_blended_collection_autolim():
     a = [1, 2, 4]
     height = .2
@@ -698,6 +715,118 @@ def test_singleton_autolim():
     np.testing.assert_allclose(ax.get_xlim(), [-0.06, 0.06])
 
 
+@pytest.mark.parametrize('flat_ref, kwargs', [
+    (True, {}),
+    (False, {}),
+    (True, dict(antialiased=False)),
+    (False, dict(transform='__initialization_delayed__')),
+])
+@check_figures_equal(extensions=['png'])
+def test_quadmesh_deprecated_signature(
+        fig_test, fig_ref, flat_ref, kwargs):
+    # test that the new and old quadmesh signature produce the same results
+    # remove when the old QuadMesh.__init__ signature expires (v3.5+2)
+    from matplotlib.collections import QuadMesh
+
+    x = [0, 1, 2, 3.]
+    y = [1, 2, 3.]
+    X, Y = np.meshgrid(x, y)
+    X += 0.2 * Y
+    coords = np.stack([X, Y], axis=-1)
+    assert coords.shape == (3, 4, 2)
+    C = np.linspace(0, 2, 6).reshape(2, 3)
+
+    ax = fig_test.add_subplot()
+    ax.set(xlim=(0, 5), ylim=(0, 4))
+    if 'transform' in kwargs:
+        kwargs['transform'] = mtransforms.Affine2D().scale(1.2) + ax.transData
+    qmesh = QuadMesh(coords, **kwargs)
+    qmesh.set_array(C)
+    ax.add_collection(qmesh)
+    assert qmesh._shading == 'flat'
+
+    ax = fig_ref.add_subplot()
+    ax.set(xlim=(0, 5), ylim=(0, 4))
+    if 'transform' in kwargs:
+        kwargs['transform'] = mtransforms.Affine2D().scale(1.2) + ax.transData
+    with pytest.warns(MatplotlibDeprecationWarning):
+        qmesh = QuadMesh(4 - 1, 3 - 1,
+                         coords.copy().reshape(-1, 2) if flat_ref else coords,
+                         **kwargs)
+    qmesh.set_array(C.flatten() if flat_ref else C)
+    ax.add_collection(qmesh)
+    assert qmesh._shading == 'flat'
+
+
+@check_figures_equal(extensions=['png'])
+def test_quadmesh_deprecated_positional(fig_test, fig_ref):
+    # test that positional parameters are still accepted with the old signature
+    # and work correctly
+    # remove when the old QuadMesh.__init__ signature expires (v3.5+2)
+    from matplotlib.collections import QuadMesh
+
+    x = [0, 1, 2, 3.]
+    y = [1, 2, 3.]
+    X, Y = np.meshgrid(x, y)
+    X += 0.2 * Y
+    coords = np.stack([X, Y], axis=-1)
+    assert coords.shape == (3, 4, 2)
+    coords_flat = coords.copy().reshape(-1, 2)
+    C = np.linspace(0, 2, 12).reshape(3, 4)
+
+    ax = fig_test.add_subplot()
+    ax.set(xlim=(0, 5), ylim=(0, 4))
+    qmesh = QuadMesh(coords, antialiased=False, shading='gouraud')
+    qmesh.set_array(C)
+    ax.add_collection(qmesh)
+
+    ax = fig_ref.add_subplot()
+    ax.set(xlim=(0, 5), ylim=(0, 4))
+    with pytest.warns(MatplotlibDeprecationWarning):
+        qmesh = QuadMesh(4 - 1, 3 - 1, coords.copy().reshape(-1, 2),
+                         False, 'gouraud')
+    qmesh.set_array(C)
+    ax.add_collection(qmesh)
+
+
+def test_quadmesh_set_array_validation():
+    x = np.arange(11)
+    y = np.arange(8)
+    z = np.random.random((7, 10))
+    fig, ax = plt.subplots()
+    coll = ax.pcolormesh(x, y, z)
+
+    # Test deprecated warning when faulty shape is passed.
+    with pytest.warns(MatplotlibDeprecationWarning):
+        coll.set_array(z.reshape(10, 7))
+
+    z = np.arange(54).reshape((6, 9))
+    with pytest.raises(TypeError, match=r"Dimensions of A \(6, 9\) "
+                       r"are incompatible with X \(11\) and/or Y \(8\)"):
+        coll.set_array(z)
+    with pytest.raises(TypeError, match=r"Dimensions of A \(54,\) "
+                       r"are incompatible with X \(11\) and/or Y \(8\)"):
+        coll.set_array(z.ravel())
+
+    x = np.arange(10)
+    y = np.arange(7)
+    z = np.random.random((7, 10))
+    fig, ax = plt.subplots()
+    coll = ax.pcolormesh(x, y, z, shading='gouraud')
+
+
+def test_quadmesh_get_coordinates():
+    x = [0, 1, 2]
+    y = [2, 4, 6]
+    z = np.ones(shape=(2, 2))
+    xx, yy = np.meshgrid(x, y)
+    coll = plt.pcolormesh(xx, yy, z)
+
+    # shape (3, 3, 2)
+    coords = np.stack([xx.T, yy.T]).T
+    assert_array_equal(coll.get_coordinates(), coords)
+
+
 def test_quadmesh_set_array():
     x = np.arange(4)
     y = np.arange(4)
@@ -714,6 +843,35 @@ def test_quadmesh_set_array():
     fig.canvas.draw()
     assert np.array_equal(coll.get_array(), np.ones(9))
 
+    z = np.arange(16).reshape((4, 4))
+    fig, ax = plt.subplots()
+    coll = ax.pcolormesh(x, y, np.ones(z.shape), shading='gouraud')
+    # Test that the collection is able to update with a 2d array
+    coll.set_array(z)
+    fig.canvas.draw()
+    assert np.array_equal(coll.get_array(), z)
+
+    # Check that pre-flattened arrays work too
+    coll.set_array(np.ones(16))
+    fig.canvas.draw()
+    assert np.array_equal(coll.get_array(), np.ones(16))
+
+
+def test_quadmesh_vmin_vmax():
+    # test when vmin/vmax on the norm changes, the quadmesh gets updated
+    fig, ax = plt.subplots()
+    cmap = mpl.cm.get_cmap('plasma')
+    norm = mpl.colors.Normalize(vmin=0, vmax=1)
+    coll = ax.pcolormesh([[1]], cmap=cmap, norm=norm)
+    fig.canvas.draw()
+    assert np.array_equal(coll.get_facecolors()[0, :], cmap(norm(1)))
+
+    # Change the vmin/vmax of the norm so that the color is from
+    # the bottom of the colormap now
+    norm.vmin, norm.vmax = 1, 2
+    fig.canvas.draw()
+    assert np.array_equal(coll.get_facecolors()[0, :], cmap(norm(1)))
+
 
 def test_quadmesh_alpha_array():
     x = np.arange(4)
@@ -775,7 +933,7 @@ def test_legend_inverse_size_label_relationship():
     assert_array_almost_equal(handle_sizes, legend_sizes, decimal=1)
 
 
-@pytest.mark.style('default')
+@mpl.style.context('default')
 @pytest.mark.parametrize('pcfunc', [plt.pcolor, plt.pcolormesh])
 def test_color_logic(pcfunc):
     z = np.arange(12).reshape(3, 4)
@@ -870,6 +1028,19 @@ def test_array_wrong_dimensions():
     pc.update_scalarmappable()
 
 
+def test_quadmesh_cursor_data():
+    fig, ax = plt.subplots()
+    *_, qm = ax.hist2d(
+        np.arange(11)**2, 100 + np.arange(11)**2)  # width-10 bins
+    x, y = ax.transData.transform([1, 101])
+    event = MouseEvent('motion_notify_event', fig.canvas, x, y)
+    assert qm.get_cursor_data(event) == 4  # (0**2, 1**2, 2**2, 3**2)
+    for out_xydata in []:
+        x, y = ax.transData.transform([-1, 101])
+        event = MouseEvent('motion_notify_event', fig.canvas, x, y)
+        assert qm.get_cursor_data(event) is None
+
+
 def test_get_segments():
     segments = np.tile(np.linspace(0, 1, 256), (2, 1)).T
     lc = LineCollection([segments])
@@ -877,3 +1048,39 @@ def test_get_segments():
     readback, = lc.get_segments()
     # these should comeback un-changed!
     assert np.all(segments == readback)
+
+
+def test_set_offsets_late():
+    identity = mtransforms.IdentityTransform()
+    sizes = [2]
+
+    null = mcollections.CircleCollection(sizes=sizes)
+
+    init = mcollections.CircleCollection(sizes=sizes, offsets=(10, 10))
+
+    late = mcollections.CircleCollection(sizes=sizes)
+    late.set_offsets((10, 10))
+
+    # Bbox.__eq__ doesn't compare bounds
+    null_bounds = null.get_datalim(identity).bounds
+    init_bounds = init.get_datalim(identity).bounds
+    late_bounds = late.get_datalim(identity).bounds
+
+    # offsets and transform are applied when set after initialization
+    assert null_bounds != init_bounds
+    assert init_bounds == late_bounds
+
+
+def test_set_offset_transform():
+    with pytest.warns(MatplotlibDeprecationWarning,
+                      match='.transOffset. without .offsets. has no effect'):
+        mcollections.Collection([],
+                                transOffset=mtransforms.IdentityTransform())
+
+    skew = mtransforms.Affine2D().skew(2, 2)
+    init = mcollections.Collection([], offsets=[], transOffset=skew)
+
+    late = mcollections.Collection([])
+    late.set_offset_transform(skew)
+
+    assert skew == init.get_offset_transform() == late.get_offset_transform()
