@@ -6,11 +6,12 @@ import functools
 import numpy as np
 import pytest
 
-from matplotlib import rc_context
+from matplotlib import rc_context, style
 import matplotlib.dates as mdates
 import matplotlib.pyplot as plt
 from matplotlib.testing.decorators import image_comparison
 import matplotlib.ticker as mticker
+import matplotlib._api as _api
 
 
 def test_date_numpyx():
@@ -69,13 +70,55 @@ def test_date2num_NaT_scalar(units):
     assert np.isnan(tmpl)
 
 
-@image_comparison(['date_empty.png'])
 def test_date_empty():
     # make sure we do the right thing when told to plot dates even
     # if no date data has been presented, cf
     # http://sourceforge.net/tracker/?func=detail&aid=2850075&group_id=80706&atid=560720
     fig, ax = plt.subplots()
     ax.xaxis_date()
+    fig.draw_without_rendering()
+    np.testing.assert_allclose(ax.get_xlim(),
+                               [mdates.date2num(np.datetime64('2000-01-01')),
+                                mdates.date2num(np.datetime64('2010-01-01'))])
+
+    mdates._reset_epoch_test_example()
+    mdates.set_epoch('0000-12-31')
+    fig, ax = plt.subplots()
+    ax.xaxis_date()
+    fig.draw_without_rendering()
+    np.testing.assert_allclose(ax.get_xlim(),
+                               [mdates.date2num(np.datetime64('2000-01-01')),
+                                mdates.date2num(np.datetime64('2010-01-01'))])
+    mdates._reset_epoch_test_example()
+
+
+def test_date_not_empty():
+    fig = plt.figure()
+    ax = fig.add_subplot()
+
+    ax.plot([50, 70], [1, 2])
+    ax.xaxis.axis_date()
+    np.testing.assert_allclose(ax.get_xlim(), [50, 70])
+
+
+def test_axhline():
+    # make sure that axhline doesn't set the xlimits...
+    fig, ax = plt.subplots()
+    ax.axhline(1.5)
+    ax.plot([np.datetime64('2016-01-01'), np.datetime64('2016-01-02')], [1, 2])
+    np.testing.assert_allclose(ax.get_xlim(),
+                               [mdates.date2num(np.datetime64('2016-01-01')),
+                                mdates.date2num(np.datetime64('2016-01-02'))])
+
+    mdates._reset_epoch_test_example()
+    mdates.set_epoch('0000-12-31')
+    fig, ax = plt.subplots()
+    ax.axhline(1.5)
+    ax.plot([np.datetime64('2016-01-01'), np.datetime64('2016-01-02')], [1, 2])
+    np.testing.assert_allclose(ax.get_xlim(),
+                               [mdates.date2num(np.datetime64('2016-01-01')),
+                                mdates.date2num(np.datetime64('2016-01-02'))])
+    mdates._reset_epoch_test_example()
 
 
 @image_comparison(['date_axhspan.png'])
@@ -196,6 +239,18 @@ def test_RRuleLocator_dayrange():
     # On success, no overflow error shall be thrown
 
 
+def test_RRuleLocator_close_minmax():
+    # if d1 and d2 are very close together, rrule cannot create
+    # reasonable tick intervals; ensure that this is handled properly
+    rrule = mdates.rrulewrapper(dateutil.rrule.SECONDLY, interval=5)
+    loc = mdates.RRuleLocator(rrule)
+    d1 = datetime.datetime(year=2020, month=1, day=1)
+    d2 = datetime.datetime(year=2020, month=1, day=1, microsecond=1)
+    expected = ['2020-01-01 00:00:00+00:00',
+                '2020-01-01 00:00:00.000001+00:00']
+    assert list(map(str, mdates.num2date(loc.tick_values(d1, d2)))) == expected
+
+
 @image_comparison(['DateFormatter_fractionalSeconds.png'])
 def test_DateFormatter():
     import matplotlib.testing.jpl_units as units
@@ -268,19 +323,23 @@ def test_date_formatter_callable():
 
 @pytest.mark.parametrize('delta, expected', [
     (datetime.timedelta(weeks=52 * 200),
-     [r'$\mathdefault{%d}$' % (year,) for year in range(1990, 2171, 20)]),
+     [r'$\mathdefault{%d}$' % year for year in range(1990, 2171, 20)]),
     (datetime.timedelta(days=30),
-     [r'Jan$\mathdefault{ %02d 1990}$' % (day,) for day in range(1, 32, 3)]),
+     [r'$\mathdefault{1990{-}01{-}%02d}$' % day for day in range(1, 32, 3)]),
     (datetime.timedelta(hours=20),
-     [r'$\mathdefault{%02d:00:00}$' % (hour,) for hour in range(0, 21, 2)]),
+     [r'$\mathdefault{01{-}01\;%02d}$' % hour for hour in range(0, 21, 2)]),
+    (datetime.timedelta(minutes=10),
+     [r'$\mathdefault{01\;00{:}%02d}$' % minu for minu in range(0, 11)]),
 ])
 def test_date_formatter_usetex(delta, expected):
+    style.use("default")
+
     d1 = datetime.datetime(1990, 1, 1)
     d2 = d1 + delta
 
     locator = mdates.AutoDateLocator(interval_multiples=False)
     locator.create_dummy_axis()
-    locator.set_view_interval(mdates.date2num(d1), mdates.date2num(d2))
+    locator.axis.set_view_interval(mdates.date2num(d1), mdates.date2num(d2))
 
     formatter = mdates.AutoDateFormatter(locator, usetex=True)
     assert [formatter(loc) for loc in locator()] == expected
@@ -319,8 +378,7 @@ def test_auto_date_locator():
     def _create_auto_date_locator(date1, date2):
         locator = mdates.AutoDateLocator(interval_multiples=False)
         locator.create_dummy_axis()
-        locator.set_view_interval(mdates.date2num(date1),
-                                  mdates.date2num(date2))
+        locator.axis.set_view_interval(*mdates.date2num([date1, date2]))
         return locator
 
     d1 = datetime.datetime(1990, 1, 1)
@@ -391,8 +449,7 @@ def test_auto_date_locator_intmult():
     def _create_auto_date_locator(date1, date2):
         locator = mdates.AutoDateLocator(interval_multiples=True)
         locator.create_dummy_axis()
-        locator.set_view_interval(mdates.date2num(date1),
-                                  mdates.date2num(date2))
+        locator.axis.set_view_interval(*mdates.date2num([date1, date2]))
         return locator
 
     results = ([datetime.timedelta(weeks=52 * 200),
@@ -556,14 +613,14 @@ def test_concise_formatter_show_offset(t_delta, expected):
       '$\\mathdefault{25}$', '$\\mathdefault{29}$', 'Feb',
       '$\\mathdefault{05}$', '$\\mathdefault{09}$']),
     (datetime.timedelta(hours=40),
-     ['Jan$\\mathdefault{{-}01}$', '$\\mathdefault{04:00}$',
-      '$\\mathdefault{08:00}$', '$\\mathdefault{12:00}$',
-      '$\\mathdefault{16:00}$', '$\\mathdefault{20:00}$',
-      'Jan$\\mathdefault{{-}02}$', '$\\mathdefault{04:00}$',
-      '$\\mathdefault{08:00}$', '$\\mathdefault{12:00}$',
-      '$\\mathdefault{16:00}$']),
+     ['Jan$\\mathdefault{{-}01}$', '$\\mathdefault{04{:}00}$',
+      '$\\mathdefault{08{:}00}$', '$\\mathdefault{12{:}00}$',
+      '$\\mathdefault{16{:}00}$', '$\\mathdefault{20{:}00}$',
+      'Jan$\\mathdefault{{-}02}$', '$\\mathdefault{04{:}00}$',
+      '$\\mathdefault{08{:}00}$', '$\\mathdefault{12{:}00}$',
+      '$\\mathdefault{16{:}00}$']),
     (datetime.timedelta(seconds=2),
-     ['$\\mathdefault{59.5}$', '$\\mathdefault{00:00}$',
+     ['$\\mathdefault{59.5}$', '$\\mathdefault{00{:}00}$',
       '$\\mathdefault{00.5}$', '$\\mathdefault{01.0}$',
       '$\\mathdefault{01.5}$', '$\\mathdefault{02.0}$',
       '$\\mathdefault{02.5}$']),
@@ -574,7 +631,7 @@ def test_concise_formatter_usetex(t_delta, expected):
 
     locator = mdates.AutoDateLocator(interval_multiples=True)
     locator.create_dummy_axis()
-    locator.set_view_interval(mdates.date2num(d1), mdates.date2num(d2))
+    locator.axis.set_view_interval(mdates.date2num(d1), mdates.date2num(d2))
 
     formatter = mdates.ConciseDateFormatter(locator, usetex=True)
     assert formatter.format_ticks(locator()) == expected
@@ -732,8 +789,7 @@ def test_auto_date_locator_intmult_tz():
     def _create_auto_date_locator(date1, date2, tz):
         locator = mdates.AutoDateLocator(interval_multiples=True, tz=tz)
         locator.create_dummy_axis()
-        locator.set_view_interval(mdates.date2num(date1),
-                                  mdates.date2num(date2))
+        locator.axis.set_view_interval(*mdates.date2num([date1, date2]))
         return locator
 
     results = ([datetime.timedelta(weeks=52*200),
@@ -951,8 +1007,8 @@ def test_yearlocator_pytz():
          + datetime.timedelta(i) for i in range(2000)]
     locator = mdates.AutoDateLocator(interval_multiples=True, tz=tz)
     locator.create_dummy_axis()
-    locator.set_view_interval(mdates.date2num(x[0])-1.0,
-                              mdates.date2num(x[-1])+1.0)
+    locator.axis.set_view_interval(mdates.date2num(x[0])-1.0,
+                                   mdates.date2num(x[-1])+1.0)
     t = np.array([733408.208333, 733773.208333, 734138.208333,
                   734503.208333, 734869.208333, 735234.208333, 735599.208333])
     # convert to new epoch from old...
@@ -966,6 +1022,45 @@ def test_yearlocator_pytz():
     assert st == expected
 
 
+def test_YearLocator():
+    def _create_year_locator(date1, date2, **kwargs):
+        locator = mdates.YearLocator(**kwargs)
+        locator.create_dummy_axis()
+        locator.axis.set_view_interval(mdates.date2num(date1),
+                                       mdates.date2num(date2))
+        return locator
+
+    d1 = datetime.datetime(1990, 1, 1)
+    results = ([datetime.timedelta(weeks=52 * 200),
+                {'base': 20, 'month': 1, 'day': 1},
+                ['1980-01-01 00:00:00+00:00', '2000-01-01 00:00:00+00:00',
+                 '2020-01-01 00:00:00+00:00', '2040-01-01 00:00:00+00:00',
+                 '2060-01-01 00:00:00+00:00', '2080-01-01 00:00:00+00:00',
+                 '2100-01-01 00:00:00+00:00', '2120-01-01 00:00:00+00:00',
+                 '2140-01-01 00:00:00+00:00', '2160-01-01 00:00:00+00:00',
+                 '2180-01-01 00:00:00+00:00', '2200-01-01 00:00:00+00:00']
+                ],
+               [datetime.timedelta(weeks=52 * 200),
+                {'base': 20, 'month': 5, 'day': 16},
+                ['1980-05-16 00:00:00+00:00', '2000-05-16 00:00:00+00:00',
+                 '2020-05-16 00:00:00+00:00', '2040-05-16 00:00:00+00:00',
+                 '2060-05-16 00:00:00+00:00', '2080-05-16 00:00:00+00:00',
+                 '2100-05-16 00:00:00+00:00', '2120-05-16 00:00:00+00:00',
+                 '2140-05-16 00:00:00+00:00', '2160-05-16 00:00:00+00:00',
+                 '2180-05-16 00:00:00+00:00', '2200-05-16 00:00:00+00:00']
+                ],
+               [datetime.timedelta(weeks=52 * 5),
+                {'base': 20, 'month': 9, 'day': 25},
+                ['1980-09-25 00:00:00+00:00', '2000-09-25 00:00:00+00:00']
+                ],
+               )
+
+    for delta, arguments, expected in results:
+        d2 = d1 + delta
+        locator = _create_year_locator(d1, d2, **arguments)
+        assert list(map(str, mdates.num2date(locator()))) == expected
+
+
 def test_DayLocator():
     with pytest.raises(ValueError):
         mdates.DayLocator(interval=-1)
@@ -1038,8 +1133,8 @@ def test_warn_notintervals():
     locator = mdates.AutoDateLocator(interval_multiples=False)
     locator.intervald[3] = [2]
     locator.create_dummy_axis()
-    locator.set_view_interval(mdates.date2num(dates[0]),
-                              mdates.date2num(dates[-1]))
+    locator.axis.set_view_interval(mdates.date2num(dates[0]),
+                                   mdates.date2num(dates[-1]))
     with pytest.warns(UserWarning, match="AutoDateLocator was unable") as rec:
         locs = locator()
 
@@ -1061,7 +1156,7 @@ def test_change_converter():
     fig.canvas.draw()
     assert ax.get_xticklabels()[0].get_text() == 'Jan 01 2020'
     assert ax.get_xticklabels()[1].get_text() == 'Jan 15 2020'
-    with pytest.warns(UserWarning) as rec:
+    with pytest.raises(ValueError):
         plt.rcParams['date.converter'] = 'boo'
 
 
@@ -1085,15 +1180,16 @@ def test_change_interval_multiples():
 
 
 def test_epoch2num():
-    mdates._reset_epoch_test_example()
-    mdates.set_epoch('0000-12-31')
-    assert mdates.epoch2num(86400) == 719164.0
-    assert mdates.num2epoch(719165.0) == 86400 * 2
-    # set back to the default
-    mdates._reset_epoch_test_example()
-    mdates.set_epoch('1970-01-01T00:00:00')
-    assert mdates.epoch2num(86400) == 1.0
-    assert mdates.num2epoch(2.0) == 86400 * 2
+    with _api.suppress_matplotlib_deprecation_warning():
+        mdates._reset_epoch_test_example()
+        mdates.set_epoch('0000-12-31')
+        assert mdates.epoch2num(86400) == 719164.0
+        assert mdates.num2epoch(719165.0) == 86400 * 2
+        # set back to the default
+        mdates._reset_epoch_test_example()
+        mdates.set_epoch('1970-01-01T00:00:00')
+        assert mdates.epoch2num(86400) == 1.0
+        assert mdates.num2epoch(2.0) == 86400 * 2
 
 
 def test_julian2num():
