@@ -50,7 +50,8 @@ def _get_xdg_cache_dir():
     """
     Return the XDG cache directory.
 
-    See https://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html
+    See
+    https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html
     """
     cache_dir = os.environ.get("XDG_CACHE_HOME")
     if not cache_dir:
@@ -62,6 +63,7 @@ def _get_xdg_cache_dir():
 
 if __name__ == "__main__":
     data = {
+        "v3.4.3": "5194481",
         "v3.4.2": "4743323",
         "v3.4.1": "4649959",
         "v3.4.0": "4638398",
@@ -104,7 +106,7 @@ if __name__ == "__main__":
     }
     doc_dir = Path(__file__).parent.parent.absolute() / "doc"
     target_dir = doc_dir / "_static/zenodo_cache"
-    citing = doc_dir / "citing.rst"
+    citing = doc_dir / "users/project/citing.rst"
     target_dir.mkdir(exist_ok=True, parents=True)
     header = []
     footer = []
@@ -132,7 +134,7 @@ if __name__ == "__main__":
             fout.write(
                 f"""
 {version}
-   .. image:: _static/zenodo_cache/{doi}.svg
+   .. image:: ../../_static/zenodo_cache/{doi}.svg
       :target:  https://doi.org/10.5281/zenodo.{doi}"""
             )
         fout.write("\n\n")
