@@ -30,6 +30,7 @@ import shutil
 import sys
 
 from matplotlib.backends.qt_compat import QtCore, QtGui, QtWidgets
+from matplotlib.backends.qt_compat import _enum, _exec
 
 
 # matplotlib stores the baseline images under two separate subtrees,
@@ -38,9 +39,9 @@ from matplotlib.backends.qt_compat import QtCore, QtGui, QtWidgets
 # these two places.
 
 BASELINE_IMAGES = [
-    os.path.join('lib', 'matplotlib', 'tests', 'baseline_images'),
-    os.path.join('lib', 'mpl_toolkits', 'tests', 'baseline_images')
-    ]
+    Path('lib/matplotlib/tests/baseline_images'),
+    Path('lib/mpl_toolkits/tests/baseline_images'),
+]
 
 
 # Non-png image extensions
@@ -61,34 +62,21 @@ class Thumbnail(QtWidgets.QFrame):
         layout = QtWidgets.QVBoxLayout()
 
         label = QtWidgets.QLabel(name)
-        label.setAlignment(QtCore.Qt.AlignHCenter |
-                           QtCore.Qt.AlignVCenter)
+        label.setAlignment(_enum('QtCore.Qt.AlignmentFlag').AlignHCenter |
+                           _enum('QtCore.Qt.AlignmentFlag').AlignVCenter)
         layout.addWidget(label, 0)
 
         self.image = QtWidgets.QLabel()
-        self.image.setAlignment(QtCore.Qt.AlignHCenter |
-                                QtCore.Qt.AlignVCenter)
-        self.image.setMinimumSize(800/3, 500/3)
+        self.image.setAlignment(_enum('QtCore.Qt.AlignmentFlag').AlignHCenter |
+                                _enum('QtCore.Qt.AlignmentFlag').AlignVCenter)
+        self.image.setMinimumSize(800 // 3, 600 // 3)
         layout.addWidget(self.image)
         self.setLayout(layout)
 
-    def mousePressEvent(self, ev):
+    def mousePressEvent(self, event):
         self.parent.set_large_image(self.index)
 
 
-class ListWidget(QtWidgets.QListWidget):
-    """
-    The list of files on the left-hand side
-    """
-    def __init__(self, parent):
-        super().__init__()
-        self.parent = parent
-        self.currentRowChanged.connect(self.change_row)
-
-    def change_row(self, i):
-        self.parent.set_entry(i)
-
-
 class EventFilter(QtCore.QObject):
     # A hack keypresses can be handled globally and aren't swallowed
     # by the individual widgets
@@ -98,11 +86,10 @@ class EventFilter(QtCore.QObject):
         self.window = window
 
     def eventFilter(self, receiver, event):
-        if event.type() == QtCore.QEvent.KeyPress:
+        if event.type() == _enum('QtCore.QEvent.Type').KeyPress:
             self.window.keyPressEvent(event)
             return True
         else:
-            return False
             return super().eventFilter(receiver, event)
 
 
@@ -120,26 +107,29 @@ class Dialog(QtWidgets.QDialog):
         event_filter = EventFilter(self)
         self.installEventFilter(event_filter)
 
-        self.filelist = ListWidget(self)
+        # The list of files on the left-hand side.
+        self.filelist = QtWidgets.QListWidget()
         self.filelist.setMinimumWidth(400)
         for entry in entries:
             self.filelist.addItem(entry.display)
+        self.filelist.currentRowChanged.connect(self.set_entry)
 
-        images_box = QtWidgets.QWidget()
-        images_layout = QtWidgets.QVBoxLayout()
         thumbnails_box = QtWidgets.QWidget()
-        thumbnails_layout = QtWidgets.QHBoxLayout()
+        thumbnails_layout = QtWidgets.QVBoxLayout()
         self.thumbnails = []
         for i, name in enumerate(('test', 'expected', 'diff')):
             thumbnail = Thumbnail(self, i, name)
             thumbnails_layout.addWidget(thumbnail)
             self.thumbnails.append(thumbnail)
         thumbnails_box.setLayout(thumbnails_layout)
+
+        images_layout = QtWidgets.QVBoxLayout()
+        images_box = QtWidgets.QWidget()
         self.image_display = QtWidgets.QLabel()
-        self.image_display.setAlignment(QtCore.Qt.AlignHCenter |
-                                        QtCore.Qt.AlignVCenter)
-        self.image_display.setMinimumSize(800, 500)
-        images_layout.addWidget(thumbnails_box, 3)
+        self.image_display.setAlignment(
+            _enum('QtCore.Qt.AlignmentFlag').AlignHCenter |
+            _enum('QtCore.Qt.AlignmentFlag').AlignVCenter)
+        self.image_display.setMinimumSize(800, 600)
         images_layout.addWidget(self.image_display, 6)
         images_box.setLayout(images_layout)
 
@@ -155,8 +145,9 @@ class Dialog(QtWidgets.QDialog):
         images_layout.addWidget(buttons_box)
 
         main_layout = QtWidgets.QHBoxLayout()
-        main_layout.addWidget(self.filelist, 3)
-        main_layout.addWidget(images_box, 6)
+        main_layout.addWidget(self.filelist, 1)
+        main_layout.addWidget(thumbnails_box, 1)
+        main_layout.addWidget(images_box, 3)
 
         self.setLayout(main_layout)
 
@@ -173,10 +164,11 @@ class Dialog(QtWidgets.QDialog):
 
         self.pixmaps = []
         for fname, thumbnail in zip(entry.thumbnails, self.thumbnails):
-            pixmap = QtGui.QPixmap(fname)
+            pixmap = QtGui.QPixmap(os.fspath(fname))
             scaled_pixmap = pixmap.scaled(
-                thumbnail.size(), QtCore.Qt.KeepAspectRatio,
-                QtCore.Qt.SmoothTransformation)
+                thumbnail.size(),
+                _enum('QtCore.Qt.AspectRatioMode').KeepAspectRatio,
+                _enum('QtCore.Qt.TransformationMode').SmoothTransformation)
             thumbnail.image.setPixmap(scaled_pixmap)
             self.pixmaps.append(scaled_pixmap)
 
@@ -184,12 +176,15 @@ class Dialog(QtWidgets.QDialog):
         self.filelist.setCurrentRow(self.current_entry)
 
     def set_large_image(self, index):
-        self.thumbnails[self.current_thumbnail].setFrameShape(0)
+        self.thumbnails[self.current_thumbnail].setFrameShape(
+            _enum('QtWidgets.QFrame.Shape').NoFrame)
         self.current_thumbnail = index
-        pixmap = QtGui.QPixmap(self.entries[self.current_entry]
-                                   .thumbnails[self.current_thumbnail])
+        pixmap = QtGui.QPixmap(os.fspath(
+            self.entries[self.current_entry]
+            .thumbnails[self.current_thumbnail]))
         self.image_display.setPixmap(pixmap)
-        self.thumbnails[self.current_thumbnail].setFrameShape(1)
+        self.thumbnails[self.current_thumbnail].setFrameShape(
+            _enum('QtWidgets.QFrame.Shape').Box)
 
     def accept_test(self):
         entry = self.entries[self.current_entry]
@@ -214,37 +209,37 @@ class Dialog(QtWidgets.QDialog):
         self.set_entry(min((self.current_entry + 1), len(self.entries) - 1))
 
     def keyPressEvent(self, e):
-        if e.key() == QtCore.Qt.Key_Left:
+        if e.key() == _enum('QtCore.Qt.Key').Key_Left:
             self.set_large_image((self.current_thumbnail - 1) % 3)
-        elif e.key() == QtCore.Qt.Key_Right:
+        elif e.key() == _enum('QtCore.Qt.Key').Key_Right:
             self.set_large_image((self.current_thumbnail + 1) % 3)
-        elif e.key() == QtCore.Qt.Key_Up:
+        elif e.key() == _enum('QtCore.Qt.Key').Key_Up:
             self.set_entry(max(self.current_entry - 1, 0))
-        elif e.key() == QtCore.Qt.Key_Down:
+        elif e.key() == _enum('QtCore.Qt.Key').Key_Down:
             self.set_entry(min(self.current_entry + 1, len(self.entries) - 1))
-        elif e.key() == QtCore.Qt.Key_A:
+        elif e.key() == _enum('QtCore.Qt.Key').Key_A:
             self.accept_test()
-        elif e.key() == QtCore.Qt.Key_R:
+        elif e.key() == _enum('QtCore.Qt.Key').Key_R:
             self.reject_test()
         else:
             super().keyPressEvent(e)
 
 
-class Entry(object):
+class Entry:
     """
     A model for a single image comparison test.
     """
     def __init__(self, path, root, source):
         self.source = source
         self.root = root
-        self.dir, fname = os.path.split(path)
-        self.reldir = os.path.relpath(self.dir, self.root)
-        self.diff = fname
+        self.dir = path.parent
+        self.diff = path.name
+        self.reldir = self.dir.relative_to(self.root)
 
-        basename = fname[:-len('-failed-diff.png')]
+        basename = self.diff[:-len('-failed-diff.png')]
         for ext in exts:
-            if basename.endswith('_' + ext):
-                display_extension = '_' + ext
+            if basename.endswith(f'_{ext}'):
+                display_extension = f'_{ext}'
                 extension = ext
                 basename = basename[:-4]
                 break
@@ -254,12 +249,11 @@ class Entry(object):
 
         self.basename = basename
         self.extension = extension
-        self.generated = basename + '.' + extension
-        self.expected = basename + '-expected.' + extension
-        self.expected_display = (basename + '-expected' + display_extension +
-                                 '.png')
-        self.generated_display = basename + display_extension + '.png'
-        self.name = os.path.join(self.reldir, self.basename)
+        self.generated = f'{basename}.{extension}'
+        self.expected = f'{basename}-expected.{extension}'
+        self.expected_display = f'{basename}-expected{display_extension}.png'
+        self.generated_display = f'{basename}{display_extension}.png'
+        self.name = self.reldir / self.basename
         self.destdir = self.get_dest_dir(self.reldir)
 
         self.thumbnails = [
@@ -267,40 +261,27 @@ class Entry(object):
             self.expected_display,
             self.diff
             ]
-        self.thumbnails = [os.path.join(self.dir, x) for x in self.thumbnails]
+        self.thumbnails = [self.dir / x for x in self.thumbnails]
 
         if not Path(self.destdir, self.generated).exists():
             # This case arises from a check_figures_equal test.
             self.status = 'autogen'
-        elif self.same(os.path.join(self.dir, self.generated),
-                     os.path.join(self.destdir, self.generated)):
+        elif ((self.dir / self.generated).read_bytes()
+              == (self.destdir / self.generated).read_bytes()):
             self.status = 'accept'
         else:
             self.status = 'unknown'
 
-    def same(self, a, b):
-        """
-        Returns True if two files have the same content.
-        """
-        return Path(a).read_bytes() == Path(b).read_bytes()
-
-    def copy_file(self, a, b):
-        """
-        Copy file from a to b.
-        """
-        print("copying: {} to {}".format(a, b))
-        shutil.copyfile(a, b)
-
     def get_dest_dir(self, reldir):
         """
         Find the source tree directory corresponding to the given
         result_images subdirectory.
         """
         for baseline_dir in BASELINE_IMAGES:
-            path = os.path.join(self.source, baseline_dir, reldir)
-            if os.path.isdir(path):
+            path = self.source / baseline_dir / reldir
+            if path.is_dir():
                 return path
-        raise ValueError("Can't find baseline dir for {}".format(reldir))
+        raise ValueError(f"Can't find baseline dir for {reldir}")
 
     @property
     def display(self):
@@ -315,36 +296,38 @@ class Entry(object):
             'autogen': '\N{WHITE SQUARE CONTAINING BLACK SMALL SQUARE}',
         }
         box = status_map[self.status]
-        return '{} {} [{}]'.format(box, self.name, self.extension)
+        return f'{box} {self.name} [{self.extension}]'
 
     def accept(self):
         """
         Accept this test by copying the generated result to the source tree.
         """
-        a = os.path.join(self.dir, self.generated)
-        b = os.path.join(self.destdir, self.generated)
-        self.copy_file(a, b)
+        copy_file(self.dir / self.generated, self.destdir / self.generated)
         self.status = 'accept'
 
     def reject(self):
         """
         Reject this test by copying the expected result to the source tree.
         """
-        a = os.path.join(self.dir, self.expected)
-        b = os.path.join(self.destdir, self.generated)
-        self.copy_file(a, b)
+        expected = self.dir / self.expected
+        if not expected.is_symlink():
+            copy_file(expected, self.destdir / self.generated)
         self.status = 'reject'
 
 
+def copy_file(a, b):
+    """Copy file from *a* to *b*."""
+    print(f'copying: {a} to {b}')
+    shutil.copyfile(a, b)
+
+
 def find_failing_tests(result_images, source):
     """
     Find all of the failing tests by looking for files with
     `-failed-diff` at the end of the basename.
     """
-    return sorted(
-        (Entry(path, result_images, source)
-         for path in Path(result_images).glob("**/*-failed-diff.*")),
-        key=lambda x: x.name)
+    return [Entry(path, result_images, source)
+            for path in sorted(Path(result_images).glob("**/*-failed-diff.*"))]
 
 
 def launch(result_images, source):
@@ -362,13 +345,13 @@ def launch(result_images, source):
     dialog.show()
     filter = EventFilter(dialog)
     app.installEventFilter(filter)
-    sys.exit(app.exec_())
+    sys.exit(_exec(app))
 
 
 if __name__ == '__main__':
     import argparse
 
-    source_dir = os.path.join(os.path.dirname(__file__), '..')
+    source_dir = Path(__file__).parent.parent
 
     parser = argparse.ArgumentParser(
         formatter_class=argparse.RawDescriptionHelpFormatter,
@@ -384,10 +367,10 @@ Keys:
     A:          Accept test.  Copy the test result to the source tree.
     R:          Reject test.  Copy the expected result to the source tree.
 """)
-    parser.add_argument("result_images", type=str, nargs='?',
-                        default=os.path.join(source_dir, 'result_images'),
+    parser.add_argument("result_images", type=Path, nargs='?',
+                        default=source_dir / 'result_images',
                         help="The location of the result_images directory")
-    parser.add_argument("source", type=str, nargs='?', default=source_dir,
+    parser.add_argument("source", type=Path, nargs='?', default=source_dir,
                         help="The location of the matplotlib source tree")
     args = parser.parse_args()
 
